import 'package:flutter/material.dart';
import 'package:fl_chart/fl_chart.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';
import 'package:gui_app/const/colors.dart';
import 'package:gui_app/const/dimens.dart';
import 'package:gui_app/provider.dart';
import 'package:intl/intl.dart';

class LegendWidget extends StatelessWidget {
  final String name;
  final Color color;

  const LegendWidget({
    Key? key,
    required this.name,
    required this.color,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final Size containerSize = MediaQuery.of(context).size;
    return Row(
      mainAxisSize: MainAxisSize.min,
      children: [
        Container(
          width: 10,
          height: 10,
          decoration: BoxDecoration(
            shape: BoxShape.circle,
            color: color,
          ),
        ),
        const SizedBox(width: 6),
        Text(
          name,
          style: TextStyle(
            color: textColor,
            fontSize: containerSize.width * chartsLegendFontSizeRatio,
          ),
        ),
      ],
    );
  }
}

class LegendsListWidget extends StatelessWidget {
  final List<Legend> legends;

  const LegendsListWidget({
    Key? key,
    required this.legends,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Wrap(
      spacing: 16,
      children: legends
          .map(
            (e) => LegendWidget(
              name: e.name,
              color: e.color,
            ),
          )
          .toList(),
    );
  }
}

class Legend {
  final String name;
  final Color color;

  Legend(this.name, this.color);
}

class Charts extends ConsumerStatefulWidget {
  final double width;
  final double height;
  const Charts(
      {Key? key, this.width = double.infinity, this.height = double.infinity})
      : super(key: key);
  @override
  ConsumerState<Charts> createState() {
    return _ChartsState();
  }
}

class _ChartsState extends ConsumerState<Charts> {
  @override
  Widget build(BuildContext context) {
    const totalCapacityColor = totalCapacityPlotColor;
    const targetColor = targetPlotColor;
    const energyGeneratedColor = energyGeneratedPlotColor;
    final formatter = NumberFormat("#W");
    final Size containerSize = MediaQuery.of(context).size;
    return SizedBox(
      width: widget.width,
      height: widget.height,
      child: Column(
        children: [
          LegendsListWidget(legends: [
            Legend(L10n.of(context)!.totalCapacity, totalCapacityColor),
            Legend(L10n.of(context)!.target, targetColor),
            Legend(L10n.of(context)!.energyGenerated, energyGeneratedColor),
          ]),
          Expanded(
            child: LineChart(
              LineChartData(
                minY: 0.0,
                maxY: 30000.0,
                // Move the x-axis first so that it does not protrude from the graph
                minX: ref.watch(totalCapacityChartProvider).first.x,
                maxX: ref.watch(totalCapacityChartProvider).last.x,
                clipData: FlClipData.horizontal(),
                titlesData: FlTitlesData(
                  topTitles:
                      AxisTitles(sideTitles: SideTitles(showTitles: false)),
                  rightTitles:
                      AxisTitles(sideTitles: SideTitles(showTitles: false)),
                  bottomTitles: AxisTitles(
                      sideTitles: SideTitles(
                    showTitles: false,
                  )),
                  leftTitles: AxisTitles(
                    sideTitles: SideTitles(
                      getTitlesWidget: (value, meta) => Container(
                        child: Text(
                          "${formatter.format(value / 1000.toInt())}",
                          style: TextStyle(
                              fontSize: containerSize.width *
                                  chartsLeftAxisFontSizeRatio,
                              color: textColor),
                        ),
                      ),
                      showTitles: true,
                    ),
                  ),
                ),
                lineBarsData: [
                  LineChartBarData(
                      spots: ref.watch(totalCapacityChartProvider),
                      dotData: FlDotData(
                        show: false,
                      ),
                      color: totalCapacityColor),
                  LineChartBarData(
                      spots: ref.watch(targetChartProvider),
                      dotData: FlDotData(
                        show: false,
                      ),
                      color: targetColor),
                  LineChartBarData(
                      spots: ref.watch(energyGeneratedChartProvider),
                      dotData: FlDotData(
                        show: false,
                      ),
                      color: energyGeneratedColor),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }
}
